.ENUM $80
Joy1Raw     DW              ; Holder of RAW joypad data from register
Joy2Raw     DW

Joy1Press   DW              ; Contains only pressed buttons (not held down)
Joy2Press   DW

Joy1Held    DW              ; Contains buttons that are Held
Joy2Held    DW

Joy1Held1	DW
Joy1HeldLong	DW
Joy1HeldLonger	DW

Joy1HeldCount	DB

.ENDE

; $4218
.EQU B_A		$80
.EQU B_X		$40
.EQU B_L		$20
.EQU B_R		$10
; $4219
.EQU B_B		$80
.EQU B_Y		$40
.EQU B_Select	$20
.EQU B_Start	$10
.EQU B_Up		$08
.EQU B_Down		$04
.EQU B_Left		$02
.EQU B_Right	$01

.BANK 0
.SECTION "JoypadCode"
;============================================================================
; Joypad -- Gets joypad data
;============================================================================
Joypad:
	php
	rep #$10
	sep #$20
	lda Joy1Press+1
	ora Joy1Held+1
	
	bit #B_Up
	beq test_down
	pha
	rep #$20
	lda sy
	sec
	sbc #$0005
	sta sy
	sep #$20
	pla
	
test_down:
	bit #B_Down
	beq test_left
	pha
	rep #$20
	lda sy
	clc
	adc #$0005
	sta sy
	sep #$20
	pla
	
test_left:	
	bit #B_Left
	beq test_right
	pha
	rep #$20
	lda sx
	clc
	adc #$0005
	sta sx
	sep #$20
	pla
test_right:
	bit #B_Right
	beq test_b
	pha
	rep #$20
	lda sx
	sec
	sbc #$0005
	sta sx
	sep #$20
	pla
	
test_b:
	bit #B_B
	beq test_y
	pha
	rep #$20
	inc scrolly
	inc scrolly
	sep #$20
	pla
test_y:
	bit #B_Y
	beq joy_low
	pha
	rep #$20
	dec scrollx
	dec scrollx
	sep #$20
	pla
		
joy_low:
	lda Joy1Press
	ora Joy1Held
	
	bit #B_A
	beq test_x
	pha
	rep #$20
	inc scrollx
	inc scrollx
	sep #$20
	pla
test_x:
	bit #B_X
	beq test_l
	pha
	rep #$20
	dec scrolly
	dec scrolly
	sep #$20
	pla	
test_l:
	bit #B_L
	beq test_r
	pha
	
	pla	
test_r:
	bit #B_R
	beq +
	pha
	
	pla
+		
	plp
	rts

UpdatePad:
    rep #$10
    sep #$20
_test1:	
	lda $4212       ; auto-read joypad status
	and #$01        ; 
	bne _test1     	; read is done when 0
	
	rep #$30        ; A/X/Y - 16 bit
	
	; Player 1
	ldx Joy1Raw		; load log of last frame's RAW read of $4218
	ldy Joy1Held
	                ; the log will be 0 the first time read of course..
	lda $4218       ; Read current frame's RAW joypad data
	sta Joy1Raw     ; save it for next frame.. (last frame log is still in X)
	txa             ; transfer last frame input from X -&gt; A (it's still in X too)
	eor Joy1Raw     ; Xor last frame input with current frame input
	                ; shows the changes in input
	                ; buttons just pressed or just released become set.
	                ; Held or unactive buttons are 0
	and Joy1Raw     ; AND changes to current frame's input.
	                ; this ends up leaving you with the only the buttons that are pressed..
	                ; It's MAGIC!
	sta Joy1Press	; Store just pressed buttons
	txa             ; Transfer last frame input from X -&gt; A again
    and Joy1Raw		; Find buttons that are still pressed (held)
    sta Joy1Held    ; by storing only buttons that are pressed both frames
    tya
    eor Joy1Held
    bne ++
    inc Joy1HeldCount
    lda Joy1HeldCount
    cmp #$04
    bne +
    lda Joy1Held
    sta Joy1HeldLong
+   cmp #$0D
    bne +++
    lda Joy1Held
    sta Joy1HeldLonger
    bra +++
++	stz Joy1HeldCount
	stz Joy1HeldLong
	stz Joy1HeldLonger
+++
    ; Player 2      ; Repeat :)
    ldx Joy2Raw
	lda $421A       ; Read Joypad2 Regs
	sta Joy2Raw
	txa
	eor Joy2Raw     ; Find  just triggered buttons
	and Joy2Raw
	sta Joy2Press
	txa
    and Joy2Raw     ; Find buttons that are still pressed (held)
    sta Joy2Held
    
    ; Joypads standard (ie not a mouse or superscope..) and connected?
    sep #$20
    ldx #$0000          ; we'll clear recorded input if pad is invalid
    
    lda $4016           ; Pad 1 - now we read this (after we stored a 0 to it earlier)
    bne _check2         ; $4016 returns 0 if not connected, 1 if connected - branch if not 0
    stx Joy1Raw         ; otherwise clear all recorded input.. it's not valid..
    stx Joy1Press
    stx Joy1Held

_check2:    
    lda $4017           ; Pad 2
    bne _done           ; 0=not connected,
    stx Joy2Raw
    stx Joy2Press
    stx Joy2Held

_done:
    RTS
    
.ENDS
